"""
Desktop group module for reading JSON files and storing their values
"""
import importlib.resources
import json
import sys

import jsonschema
from desktop_groups import assets


class Group:
    """
    A desktop group
    """
    def __init__(self, name: str, icon: str = None):
        """
        :param name: Group name
        :param icon: Group icon
        """

        self.name = name
        self.icon = icon

        self.items = []

    def add_item(self, name: str, icon: str, command: str):
        """
        Adds an item to the items list
        :param name: Item name
        :param icon: Item icon
        :param command: Item command
        """

        self.items.append(dict(name = name, icon = icon, command = command))

    def remove_item(self, name: str):
        """
        Removes an item from the items list
        :param name: Item name
        :return:
        """

        for item in self.items:
            if item['name'] == name:
                self.items.remove(item)
                break


class DGFileGroup(Group):
    """
    A desktop group created from a file
    """
    def __init__(self, dg_file: str):
        """
        :param dg_file: Location of file
        """

        # Read desktop group file
        with open(dg_file, 'r', encoding='utf-8') as file:
            self.data = json.load(file)

        # Read JSON schema
        inp_file = importlib.resources.files(assets) / 'json/desktopgroups.schema.json'
        with inp_file.open('rt') as schema:
            self.schema = json.load(schema)

        # Validate JSON
        if not self.validate():
            print('Error while validating JSON')
            sys.exit(1)

        # Initialize Group class
        super().__init__(self.data.get('group', {}).get('name'), self.data.get('group', {}).get('icon'))

        # Add items to self.items list
        group = self.data.get('group', {})
        items = group.get('items', [])

        for item in items:
            self.items.append(item)

    def validate(self):
        """
        Validates the desktop group file
        :return: Returns True if file is valid
        """

        try:
            jsonschema.validate(self.data, self.schema)
        except jsonschema.ValidationError as e:
            print(f'Error: {e}')
            return False
        return True
